#include <Wire.h>

#include <Adafruit_SSD1306.h>

Adafruit_SSD1306 display(128, 64, &Wire, -1);

void setup() {
  Serial.begin(9600);
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.setTextSize(2);
  display.setTextColor(WHITE);
}

void initNum(byte shift) {
  display.fillRect(shift, 8, 30, 48, 1);
}

void draw0(byte shift) {
  initNum(shift);
  display.fillRect(shift + 6, 14, 18, 36, 0);
}

void draw1(byte shift) {
  display.fillRect(shift + 6, 8, 18, 48, 1);
  display.fillRect(shift + 6, 14, 6, 36, 0);
  display.fillRect(shift + 18, 8, 6, 42, 0);
}

void draw2(byte shift) {
  initNum(shift);
  display.fillRect(shift, 14, 24, 15, 0);
  display.fillRect(shift + 6, 35, 24, 15, 0);
}

void draw3(byte shift) {
  draw0(shift);
  display.fillRect(shift + 6, 29, 18, 6, 1);
  display.fillRect(shift, 14, 6, 36, 0);
}

void draw4(byte shift) {
  draw8(shift);
  display.fillRect(shift + 6, 8, 18, 6, 0);
  display.fillRect(shift, 35, 24, 21, 0);
}

void draw5(byte shift) {
  draw6(shift);
  display.fillRect(shift, 35, 6, 15, 0);
}

void draw6(byte shift) {
  draw8(shift);
  display.fillRect(shift + 24, 14, 6, 15, 0);
}

void draw7(byte shift) {
  draw8(shift);
  display.fillRect(shift, 26, 24, 30, 0);
}

void draw8(byte shift) {
  initNum(shift);
  display.fillRect(shift + 6, 14, 18, 15, 0);
  display.fillRect(shift + 6, 35, 18, 15, 0);
}

void draw9(byte shift) {
  draw8(shift);
  display.fillRect(shift, 35, 6, 15, 0);
}

void drawDigit(byte num, byte shift) {
  switch (num) {
    case 0:
      draw0(shift);
      break;
    case 1:
      draw1(shift);
      break;
    case 2:
      draw2(shift);
      break;
    case 3:
      draw3(shift);
      break;
    case 4:
      draw4(shift);
      break;
    case 5:
      draw5(shift);
      break;
    case 6:
      draw6(shift);
      break;
    case 7:
      draw7(shift);
      break;
    case 8:
      draw8(shift);
      break;
    case 9:
      draw9(shift);
      break;
    default:
      break;
  }
}

void drawNum(short num) {
  if (0 <= num && num <= 9) {
    drawDigit((byte) num, 49);
  }
  else if (10 <= num && num <= 99) {
    drawDigit((byte) num / 10, 33);
    drawDigit((byte) num % 10, 65);
  }
  else if (100 <= num && num <= 999) {
    drawDigit((byte) (num / 100), 17);
    drawDigit((byte) ((num % 100) / 10), 49);
    drawDigit((byte) (num % 10), 81);
  }
  else {
    drawDigit((byte) (num / 1000), 1);
    drawDigit((byte) ((num % 1000) / 100), 33);
    drawDigit((byte) ((num % 100) / 10), 65);
    drawDigit((byte) (num % 10), 97);
  }
}

short rec_val;
byte b1, b2;
byte gearNum;
byte pageNum, screenNum;
uint64_t last_menu_time;
const uint64_t menu_timeout = 3000; // milliseconds

void loop() {
  if (2 <= Serial.available()) {
    b1 = Serial.read();
    b2 = Serial.read();
    rec_val = (b2 << 8) + b1;
  }

  if (rec_val >= 1000) {
    last_menu_time = millis();
    pageNum = (rec_val % 100) / 10;
    screenNum = rec_val % 10;
  }
  else {
    gearNum = rec_val / 100;
  }

  display.clearDisplay();
  if (millis() - last_menu_time < menu_timeout) {
    display.setCursor(0, 0);
    display.print("SCREEN: ");
    display.println(screenNum + 1);
    display.print("\nPAGE:   ");
    display.println(pageNum + 1);
  }
  else {
    drawNum(gearNum);
  }
  display.display();
}
